/***************************************************************************\
*                                                                           *
* BuilderX Version 4.0                                                      *
*                                                                           *
* Copyright (C) 1999 Xironic.                                               *
*                                                                           *
\***************************************************************************/

//---------------------------------------------------------------------------
#ifndef BDX3DViewH
#define BDX3DViewH
//---------------------------------------------------------------------------
#include "BX3DFrame.h"
#include "BX3DCamera.h"
#include "BX3DLight.h"
#include "BX3DObject.h"
#include "BX3DScene.h"
//---------------------------------------------------------------------------

TBX3DCustomFrame* GetFirstScene(TBX3DCustomFrame*);
LPDIRECT3DRMVIEWPORT2 GetViewport(TBX3DCustomFrame*);
typedef void __fastcall (__closure *TFlipError)(System::TObject* Sender,HRESULT Error);
typedef void __fastcall (__closure *TDrawEvent)(System::TObject* Sender, HDC hDC);
typedef void __fastcall (__closure *TPickedEvent)(System::TObject* Sender, TObject* PickedObject,int Layer);

enum TDisplayMode{dm640x480x16,
                  dm320x200x16,dm320x200x24,dm320x200x32,
                  dm320x240x16,dm320x240x24,dm320x240x32,
                  dm400x300x16,dm400x300x24,dm400x300x32,
                  dm512x384x16,dm512x384x24,dm512x384x32,
                  dm640x400x16,dm640x400x24,dm640x400x32,
                  dm640x480x24,dm640x480x32,
                  dm800x600x16,dm800x600x24,dm800x600x32,
                  dm1024x768x16,dm1024x768x24,dm1024x768x32,
                  dm1152x864x16,dm1152x864x24,dm1152x864x32,
                  dm1280x1024x16,dm1280x1024x24,dm1280x1024x32,
                  dm1600x1200x16,dm1600x1200x24,dm1600x1200x32
                 };

enum TRShadeMode{smGouraud,smFlat,smPhong};
enum TRFillMode{fmSolid,fmWireframe,fmPoints};
enum TTextureQuality{tqNearest,tqLinear,tqMipNearest,tqMipLinear,tqLinearMipNearest,tqLinearMipLinear};
enum TProjectionType{ptPerspective,ptOrthographic};
enum TProjectionHand{phLeftHand,phRightHand};
enum TDriverType{dtRGB,dtHAL};
enum TObjectStorage{osFrame,osObject,osLight,osScene,osCamera};
enum TSetRM{rmBlendedTransparency,rmSortedTransparency,rmLightInModelSpace,rmDisableSortedAlphaZWrite,rmViewDependentSpecular};
typedef Set<TSetRM,rmBlendedTransparency,rmViewDependentSpecular> TRenderMode;

//TBXDeviceInfo
//---------------------------------------------------------------------------
class TBXDeviceInfo
{

  private:

  public:
    GUID Guid;
    AnsiString Name;
    AnsiString Desc;
    D3DDEVICEDESC D3DHalDesc;
    D3DDEVICEDESC D3DHelDesc;
};

//TBXDrivers
//---------------------------------------------------------------------------
class TBXDrivers
{

  private:

  public:
    TBXDeviceInfo* DriverHAL;
    TBXDeviceInfo* DriverRGB;
    __fastcall TBXDrivers();
};

//TBX3DPlane
//---------------------------------------------------------------------------
class PACKAGE TBX3DPlane : public TPersistent
{
  private:
    TNotifyEvent FOnChange;
    float FBack;
    float FFront;
    float FLeft;
    float FRight;
    float FTop;
    float FBottom;
    void __fastcall SetBack(float);
    void __fastcall SetFront(float);
    void __fastcall SetLeft(float);
    void __fastcall SetRight(float);
    void __fastcall SetTop(float);
    void __fastcall SetBottom(float);
    float FField;
    void __fastcall SetField(float);

  public:
    __fastcall TBX3DPlane();

  __published:
    __property float Back={read=FBack,write=SetBack,default=5000};
    __property float Front={read=FFront,write=SetFront,default=1};
    __property float Left={read=FLeft,write=SetLeft};
    __property float Right={read=FRight,write=SetRight};
    __property float Top={read=FTop,write=SetTop};
    __property float Bottom={read=FBottom,write=SetBottom};
    __property float Field={read=FField,write=SetField,default=(1/2)};
    __property TNotifyEvent OnChange={read=FOnChange,write=FOnChange};
};

class TRenderLoop;
class TBX3DView;

//TBX3DViewQuality
//---------------------------------------------------------------------------
class PACKAGE TBX3DViewQuality : public TPersistent
{
  private:
    TBX3DView* OwnerView;
    TRShadeMode FShadeMode;
    bool FLightMode;
    TRFillMode FFillMode;
    void __fastcall SetShadeMode(TRShadeMode);
    void __fastcall SetLightMode(bool);
    void __fastcall SetFillMode(TRFillMode);
    bool FDither;
    void __fastcall SetDither(bool);
    bool __fastcall GetDither();
    TTextureQuality FTextureQuality;
    void __fastcall SetTextureQuality(TTextureQuality);
    TTextureQuality __fastcall GetTextureQuality();
    void __fastcall Changing();

  public:
    void __fastcall ChangeQuality();
    __fastcall TBX3DViewQuality(TBX3DView*);

  __published:
    __property TRShadeMode ShadeMode={read=FShadeMode,write=SetShadeMode};
    __property bool LightMode={read=FLightMode,write=SetLightMode};
    __property TRFillMode FillMode={read=FFillMode,write=SetFillMode};
    __property bool Dither={read=GetDither,write=SetDither};
    __property TTextureQuality TextureQuality={read=GetTextureQuality,write=SetTextureQuality};
};

//TDisplayRec
//---------------------------------------------------------------------------
class TDisplayRec : public TObject
{
  public:
   __fastcall TDisplayRec();
   DWORD Width;
   DWORD Height;
   DWORD BPP;
};

//TBX3DView
//---------------------------------------------------------------------------
class PACKAGE TBX3DView : public TCustomControl
{
  private:

    DDPIXELFORMAT ZBuf_ddpf;
    float FRenderDelta;
    int __fastcall GetWidth();
    void __fastcall SetWidth(int);
    int __fastcall GetHeight();
    void __fastcall SetHeight(int);
    TDisplayRec* DisplayModeRec;
    TDisplayMode FDisplayMode;
    void __fastcall SetDisplayMode(TDisplayMode);
    bool __fastcall CheckDisplayModeValid(AnsiString);
    bool FFullScreen;
    void __fastcall SetFullScreen(bool);
    bool FFullScreenSwitch;
    void __fastcall SetFullScreenSwitch(bool);
    TPickedEvent FOnPicked;
    TBX3DViewQuality* FQuality;
    void __fastcall SetQuality(TBX3DViewQuality*);

    bool wasAutoRender;
    bool FAutoRender;
    void __fastcall SetAutoRender(bool);
    void __fastcall RunRender(bool);

    int FFPS;
    int __fastcall GetFPS();
    TBX3DScene* FScene;
    void __fastcall SetScene(TBX3DScene*);
    TBX3DCamera* FCamera;
    void __fastcall SetCamera(TBX3DCamera*);
    TFlipError FOnFlipError;
    TNotifyEvent FOnFlip;
    TDrawEvent FOnDrawBackGDI;
    TDrawEvent FOnDrawFrontGDI;
    TNotifyEvent FOnPaint;
    MESSAGE void __fastcall WMPaint(TWMPaint &Message);
    TNotifyEvent FOnResize;
    MESSAGE void __fastcall WMSize(TWMSize &Message);
    bool FShowBackFaces;
    void __fastcall SetShowBackFaces(bool);
    void __fastcall ChangeShowBackFaces();
    TRenderMode FRenderMode;
    void __fastcall SetRenderMode(TRenderMode);
    TRenderMode __fastcall GetRenderMode();
    void __fastcall ChangeRenderMode();
    bool __fastcall InitApp();
    void __fastcall  Change();
    void __fastcall FindTopParentFrame(LPDIRECT3DRMFRAME);
    TBX3DPlane* FPlane;
    void __fastcall SetPlane(TBX3DPlane*);
    void __fastcall ChangePlane(TObject*);
    TProjectionType FProjectionType;
    void __fastcall SetProjectionType(TProjectionType);
    TProjectionType __fastcall GetProjectionType();
    TProjectionHand FProjectionHand;
    void __fastcall SetProjectionHand(TProjectionHand);
    TProjectionHand __fastcall GetProjectionHand();
    bool FUniformScaling;
    void __fastcall SetUniformScaling(bool);
    bool AllowResize;
    void __fastcall Resize();
    void __fastcall Paint();
    void __fastcall Activate(WORD);
    LPDIRECT3DRMDEVICE3 FIDirect3DRMDevice3;
    LPDIRECT3DRMDEVICE3 __fastcall GetIDirect3DRMDevice3();
    LPDIRECTDRAWCLIPPER FIDirect3DClipper;
    LPDIRECTDRAWCLIPPER __fastcall GetIDirect3DClipper();
    LPDIRECT3DRMVIEWPORT2 FIDirect3DRMViewport2;
    LPDIRECT3DRMVIEWPORT2 __fastcall GetIDirect3DRMViewport2();
    AnsiString FStorageFile;
    MESSAGE void __fastcall WMCreate(TWMCreate &Message);
    bool FAutoCheckCollision;
    TStringList* FAvailableModes;
    
  public:
    virtual void __fastcall Notification(TComponent* AComponent,TOperation Operation);
    HRESULT rCV;
    bool ForceRGB;
    bool __fastcall CanGet();

    TRenderLoop* RenderLoop;
    void __fastcall CallOnDrawBackGDI();
    void __fastcall CallOnDrawFrontGDI();
    bool __fastcall Flip();
    void __fastcall CallFlip();
    void __fastcall CallOnFlip();
    HWND DDWndHandle;
    LPDIRECTDRAWSURFACE4 FIDirectPrimarySurface4;
    LPDIRECTDRAWSURFACE4 FIDirectBackSurface4;
    LPDIRECTDRAWSURFACE4 FIDirectZBufferSurface4;
    LPDIRECT3DDEVICE2 FIDirect3DDevice2;
    LPDIRECT3DDEVICE3 FIDirect3DDevice3;
    LPDIRECTDRAW4 FIDirectDraw4;
    LPDIRECT3D3 FIDirect3D3;
    LPDIRECT3D2 FIDirect3D2;
    LPDIRECT3D FIDirect3D;
    int rw,rh;
    RECT g_rcRestoreControl;
    RECT g_rcRestoreWindow;
    RECT g_rcWindow;
    RECT g_rcViewport;
    RECT g_rcScreen;
    void __fastcall ViewKeyDown(TObject* Sender, WORD &Key,TShiftState Shift);
    void __fastcall SubClassWndProc(Messages::TMessage &Message);
    TAlign PrevAlign;
    void __fastcall UpdatePos();
    AnsiString __fastcall GetUniqueName(TObjectStorage);
    void __fastcall ReadStorage();
    void __fastcall WriteStorage();
    TBX3DFrame* __fastcall AddToStorage(TObjectStorage);
    TBX3DFrame* __fastcall AddToStorage(TBX3DFrame*);
    void __fastcall ClearStorage();
    void __fastcall DeleteFromStorage(TBX3DFrame*);
    void __fastcall DeleteFromStorage(AnsiString);
    bool NeedChange;
    bool ChangeViewportFullScreen;
    void __fastcall CallChangeViewport();
    HRESULT __fastcall ChangeViewport();
    bool __fastcall CV2();
    HWND FCurrentHandle;
    HWND __fastcall GetCurrentHandle();
    bool __fastcall RenderIM();
    TDriverType FDriverType;
    void __fastcall SetDriverType(TDriverType);
    TList* FPickedList;
    DWORD tick;
    DWORD prevsec;
    DWORD prevsec2;
    int cnt;
    bool Running;
    void __fastcall Update3D();
    void __fastcall UpdateEBR();
    void __fastcall UpdateEAR();
    void __fastcall CheckObjects(TBX3DObject*,TBX3DObject*);

//methods for client
    void __fastcall CollisionChecking();
    void __fastcall Render();
    AnsiString __fastcall GetDisplayMode();
    bool __fastcall DisplayModeStr(AnsiString);
    void __fastcall GenPick(int X,int Y);

    __fastcall TBX3DView(TComponent* Owner);
    __fastcall ~TBX3DView();

    __property LPDIRECTDRAWCLIPPER IDirect3DClipper={read=GetIDirect3DClipper};
    __property LPDIRECT3DRMDEVICE3 IDirect3DRMDevice3={read=GetIDirect3DRMDevice3,write=FIDirect3DRMDevice3};
    __property LPDIRECT3DRMVIEWPORT2 IDirect3DRMViewport2={read=GetIDirect3DRMViewport2,write=FIDirect3DRMViewport2};
    __property Canvas;
    __property HWND CurrentHandle={read=GetCurrentHandle};
    __property bool FullScreen={read=FFullScreen,write=SetFullScreen};
    __property TStringList* AvailableModes={read=FAvailableModes};
    __property TList* PickedList={read=FPickedList};
  __published:
    __property TBX3DScene* Scene={read=FScene,write=SetScene};
    __property float RenderDelta={read=FRenderDelta,write=FRenderDelta,default=1};
    __property bool AutoCheckCollision={read=FAutoCheckCollision,write=FAutoCheckCollision};
    __property int FPS={read=GetFPS,write=FFPS};
    __property int Width={read=GetWidth,write=SetWidth};
    __property int Height={read=GetHeight,write=SetHeight};
    __property TDisplayMode DisplayMode={read=FDisplayMode,write=SetDisplayMode};
    __property TFlipError OnFlipError={read=FOnFlipError,write=FOnFlipError};
    __property TNotifyEvent OnFlip={read=FOnFlip,write=FOnFlip};
    __property TDrawEvent OnDrawBackGDI={read=FOnDrawBackGDI,write=FOnDrawBackGDI};
    __property TDrawEvent OnDrawFrontGDI={read=FOnDrawFrontGDI,write=FOnDrawFrontGDI};
    __property TPickedEvent OnPicked={read=FOnPicked,write=FOnPicked};
    __property TProjectionType ProjectionType={read=GetProjectionType,write=SetProjectionType};
    __property TProjectionHand ProjectionHand={read=GetProjectionHand,write=SetProjectionHand};
    __property bool UniformScaling={read=FUniformScaling,write=SetUniformScaling,default=true};
    __property TBX3DPlane* Plane={read=FPlane,write=SetPlane, nodefault};
    __property TBX3DViewQuality* Quality={read=FQuality,write=SetQuality};
    __property TDriverType DriverType={read=FDriverType,write=SetDriverType};
    __property TRenderMode RenderMode={read=GetRenderMode,write=SetRenderMode};
    __property bool ShowBackFaces={read=FShowBackFaces,write=SetShowBackFaces};
    __property TNotifyEvent OnPaint={read=FOnPaint,write=FOnPaint};
    __property TNotifyEvent OnResize={read=FOnResize,write=FOnResize};
    __property bool AutoRender={read=FAutoRender,write=SetAutoRender,nodefault};
    __property TBX3DCamera* Camera={read=FCamera,write=SetCamera};
    __property Align;
    __property OnClick;
    __property OnDblClick;
    __property OnMouseDown;
    __property OnMouseMove;
    __property OnMouseUp;
    __property MouseCapture;

#if __BORLANDC__==0x520
  BEGIN_MESSAGE_MAP
    MESSAGE_HANDLER(WM_PAINT,TWMPaint, WMPaint);
    MESSAGE_HANDLER(WM_SIZE, TWMSize, WMSize);
    MESSAGE_HANDLER(WM_CREATE, TWMCreate, WMCreate);
  END_MESSAGE_MAP(TCustomControl);
#else
  BEGIN_MESSAGE_MAP
    VCL_MESSAGE_HANDLER(WM_PAINT,TWMPaint, WMPaint);
    VCL_MESSAGE_HANDLER(WM_SIZE, TWMSize, WMSize);
    VCL_MESSAGE_HANDLER(WM_CREATE, TWMCreate, WMCreate);
  END_MESSAGE_MAP(TCustomControl);
#endif
};

//TRenderLoop
//---------------------------------------------------------------------------
class TRenderLoop: public TThread
{
  private:
    TBX3DView* ptD3D;
    void __fastcall UpdateView();
    void __fastcall Stop();
    LPDIRECT3DRMDEVICE3 Device;
    LPDIRECT3DRMVIEWPORT2 Viewport;
    LPDIRECT3DRMFRAME3 Scene;

  protected:
    virtual void __fastcall Execute(void);

  public:
    bool DoRender;
    bool PrepSuspend;
    bool InSuspendLoop;
    AnsiString MsgFrom;
    bool CallChangeViewport;
    bool Term;
    bool Run;
    DWORD tick;
    DWORD prevsec;
    DWORD prevsec2;
    int cnt;
    int fpsc;
    int fpsm;
    void __fastcall UpdateParam();
    void __fastcall ReleaseVD();
    __fastcall TRenderLoop(TBX3DView*);
    __fastcall ~TRenderLoop();
};

//---------------------------------------------------------------------------
#endif
